/*
 *	smtp-gated.h
*/


#ifndef _SMTP_GATED_H_
#define _SMTP_GATED_H_

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <err.h>
#include <syslog.h>
#include <signal.h>

#include "setup.h"

#ifndef _SMTP_GATED_C_
#define EXTERN extern
#else
#define EXTERN
#endif

/*
 *	hard configuration
*/

// don't change!!
#define RCPTS_ONE_TIME		2

/*
 *	ident results
*/

#define LOOKUP_OK		0x00
#define LOOKUP_TIMEOUT		0x01
#define LOOKUP_MISMATCH		0x02
#define LOOKUP_NOMEM		0x03


#define LOG_MAIL_ACCEPTED	0x01
#define LOG_MAIL_REJECTED	0x02
#define LOG_MAIL_BASE64		0x04


/*
 * 	macros
*/

#define SET_TIMEOUT(x)		timedout = 0; alarm(x);
#define CLEAR_TIMEOUT()		timedout = 0; alarm(0);




/*
 *	SMTP commands (~verbs)
*/

typedef enum { COMMAND_NONE = 0, COMMAND_OTHER, COMMAND_HELO, COMMAND_EHLO, COMMAND_STARTTLS, COMMAND_RSET,
	COMMAND_MAIL, COMMAND_RCPT, COMMAND_DATA, COMMAND_DATA_ACK, COMMAND_BDAT, COMMAND_QUIT } smtp_command;



/*
 *	SMTP DATA state
*/
typedef enum { GOING_NONE = 0, GOING_HEADER, GOING_BODY } data_phase;

/*
 *	connections states (conn_states[] indesex)
*/
typedef enum { CONN_START = 0, CONN_IDENT, CONN_CONNECT, CONN_PRE, CONN_DATA, CONN_BDAT,
	CONN_DIRECT, CONN_SCAN, CONN_SCAN1, CONN_SCAN2, CONN_SPAM, CONN_SPAM1, CONN_SPAM2,
	CONN_POST, CONN_QUIT } conn_state;


/*
 *	SMTP response struct
*/
struct response_code {
	int generic;		// 250
	int subject;		// 1
	int detail;		// 0
};


/*
 *	pipeline queue entry
*/
typedef uint command_parm_t;

struct command_entry {
	smtp_command cmd;
	command_parm_t parm1;	/* BDAT bytes to go */
	command_parm_t parm2;	/* BDAT LAST */
};


/*
 *	session data
*/
struct proc_data {
	// connections fd
	int client;			// (from) client socket
	int server;			// (to) server socket

	// connection data
	char ident[IDENT_SIZE+1];	// remote size ident string

	struct sockaddr_in origin;	// source (client/NAT) address
	char origin_str[16];		// source (client/NAT) IP text-address
	int origin_port;		// source (client/NAT) port
	
	struct sockaddr_in target;	// target (SMTP server) address
	char target_str[16];		// target (SMTP server) IP text-address
	int target_port;		// target (SMTP server) port

	struct sockaddr_in local_addr;	// source -> local address data
//	socklen_t local_addr_len;	// local_addr struct size
	
	// buffers
	char *cli_buf;			// bufor danych od klienta
	char *srv_buf;			// bufor danych z serwera
	int cli_size;			// ilosc danych w buforach
	int srv_size;
	
	// header data
	union {
		// uzywane podczas normalnej sesji
		struct {
			int command_pos_client;		// queue position for client
			int command_pos_server;		// queue position for server
			struct command_entry commands[PIPELINE_QUEUE];	// command pipeline queue

		};

		// uzywane tylko przez wait_for_quit
		struct {
			char *message;
			int command_count;
		};
	};


	// nat header, to be injected to message
	char *header;			// nat header
	int header_size;		// nat header length

	// spooling data
	size_t size;			// spool size
	size_t extra_size;			// injected Spool-Info header size
	char *spool_name;		// spool file name
	int spool_fd;			// spool file descriptor || -1
	int spool_exists;		// spool file created && !deleted

	// lock file
	char *lockfile;		// lock filename

	// extension support
	int enhancedstatuscodes;	// ENHANCEDSTATUSCODES
	int chunking;			// CHUNKING

	// RFC 3030, BDAT n [LAST]
	int data_used;			// DATA issued (per transaction)
	int bdat_used;			// BDAT issued (per transaction)
	int bdat_last;			// last chunk indicator [flag]
	int bdat_togo;			// bytes left to be read

	// DATA state
	data_phase data_going;		// DATA going

	// virus name
	char *virus_name;		// if found or NULL

	// recipients
	char *mail_from;		// pointer to mail from address
	char *rcpt_to[RCPTS_ONE_TIME];		// pointers to mail recipients
	int rcpt_to_code[RCPTS_ONE_TIME];	// rcpt[i] is accept code by MTA
	int rcpts;			// recipients in buffer

	// statistics
	int cli_rx, srv_rx;		// statistics byte-counters
	int rcpts_total;		// recipients count
	int transaction;		// SMTP transaction no.
	time_t time;			// start time

	// uzywana tylko przy USE_SHARED_MEM, ale jest w logach wiec zostaje
	int ident_count;		// count clients from this ident (not IP)
};


/*
 *	connection info (statistics)
*/
struct conn_info {
	pid_t pid;
	uint32_t src;
	int start_time;
#ifdef USE_SHARED_MEM
	int ident_ok;
	char ident[IDENT_SIZE+1];
	uint32_t dst;
	int cli_rx, srv_rx;
	int transaction;
	conn_state state;
#endif
};


/*
 *	global statistics
*/
struct stat_info {
	time_t started;
	time_t restarted;

	int max_children;

	int requests;
	int rejects;
	int rejects_host;
	int rejects_lock;

	// shared_mem variables
	int rejects_ident;
	int viruses;
	int spams;
	int requests_direct;
	int requests_empty;
};



#ifndef _SMTP_GATED_C_
EXTERN volatile sig_atomic_t timedout;
EXTERN int i_am_a_child;
#endif


EXTERN void action(int prio, char *format, ...)
	__attribute__ ((format (printf, 2, 3)));

EXTERN int fdprintf(int fd, char *format, ...)
	__attribute__ ((format (printf, 2, 3)));

EXTERN void set_dump_state(conn_state s);
	

#undef EXTERN

#endif


